# ==============================================================================
#   IMPORTS
# ==============================================================================
# Python Imports
import os
import time
import cPickle

# EventScripts Imports
import es
import gungamelib

# ==============================================================================
#   GLOBALS
# ==============================================================================
dict_knifers = {}
        
# ==============================================================================
#  KNIFERS CLASS
# ==============================================================================
class Knifers(object):
    '''Class used for tracking and storing Knifers'''
    
    def __init__(self, uniqueid):
        self.uniqueid = str(uniqueid)
        
        # Load database
        if not gungamelib.getGlobal('knifersloaded'):
            loadKniferDatabase()
        
        if self.uniqueid not in dict_knifers:
            self.attributes = {'kills': 0, 'deaths': 0, 'timestamp': time.time(), 'name': '<UNKNOWN>'}
            dict_knifers[self.uniqueid] = self.attributes
        else:
            self.attributes = dict_knifers[self.uniqueid]
    
    def __getitem__(self, item):
        # Make the item a lower-case string
        item = str(item).lower()
        
        # Does the attribute exist?
        if item not in self.attributes:
            raise KeyError(item)
        
        return self.attributes[item]
    
    def __setitem__(self, item, value):
        # Make the item a lower-case string
        item = str(item).lower()
        
        # Does the attribute exist?
        if item not in self.attributes:
            return
        
        if item == 'kills':
            self.attributes[item] = int(value)
        elif item == 'deaths':
            self.attributes[item] = int(value)
        elif item == 'timestamp':
            self.attributes[item] = float(value)
        else:
            self.attributes[item] = value

#-------------------------------------------------
            
def getKnifer(uniqueid):
    return Knifers(uniqueid)
    
# ==============================================================================
#   KNIFER RELATED COMMANDS
# ==============================================================================
def getKniferList():
    return dict_knifers.keys()

def getTotalKnifers():
    return len(getKniferList())

def getKniferRank(steamid):
    knifers = getOrderedKnifers()
    
    # Not a kniffer?
    if steamid not in knifers:
        return -1
    
    return knifers.index(steamid)+1

def getOrderedKnifers():
    return sorted(dict_knifers, key=lambda x: dict_knifers[x]['kills']-dict_knifers[x]['deaths'], reverse=True)

def getKniferName(uniqueid):
    if uniqueid in dict_knifers:
        return dict_knifers[uniqueid]['name']
    else:
        return '<UNKNOWN>'

def getKnifeKills(uniqueid):
    uniqueid = str(uniqueid)
    
    if uniqueid in dict_knifers:
        return dict_knifers[uniqueid]['kills']
    else:
        return 0

def getKnifeDeaths(uniqueid):
    uniqueid = str(uniqueid)
    
    if uniqueid in dict_knifers:
        return dict_knifers[uniqueid]['deaths']
    else:
        return 0

def addKnifeKill(uniqueid):
    gungameKnifer = getKnifer(uniqueid)
    gungameKnifer['kills'] += 1
    gungameKnifer['name'] = es.getplayername(es.getuserid(uniqueid))

def addKnifeDeath(uniqueid):
    gungameKnifer = getKnifer(uniqueid)
    gungameKnifer['deaths'] += 1
    gungameKnifer['name'] = es.getplayername(es.getuserid(uniqueid))

def updateTimeStamp(uniqueid):
    gungameKnifer = getKnifer(uniqueid)
    gungameKnifer['timestamp'] = time.time()

def saveKniferDatabase():
    global dict_knifers
    
    # Set the knifers database path to a variable
    knifersDataBasePath = es.getAddonPath('gungame') + '/data/knifersdata.db'
    
    # Dump to the file
    knifersDataBaseFile = open(knifersDataBasePath, 'w')
    cPickle.dump(dict_knifers, knifersDataBaseFile)
    knifersDataBaseFile.close()

def loadKniferDatabase():
    global dict_knifers
    
    # Set a variable for the path of the knifer's database
    knifersDataBasePath = es.getAddonPath('gungame') + '/data/knifersdata.db'
    
    # See if the file exists
    if not os.path.isfile(knifersDataBasePath):
        # Open the file
        knifersDataBaseFile = open(knifersDataBasePath, 'w')
        
        # Place the contents of dict_knifers in
        cPickle.dump(dict_knifers, knifersDataBaseFile)
        
        # Save changes
        knifersDataBaseFile.close()
    
    # Read the file
    knifersDataBaseFile = open(knifersDataBasePath, 'r')
    dict_knifers = cPickle.load(knifersDataBaseFile)
    knifersDataBaseFile.close()
    
    # Set the global for having the database loaded
    gungamelib.setGlobal('knifersloaded', 1)

def pruneKniferDatabase(days):
    daysInSeconds = float(days) * float(86400)
    currentTime = float(time.time())
    
    for steamid in dict_knifers.copy():
        # Prune the player?
        if (currentTime - float(dict_knifers[steamid]['timestamp'])) > daysInSeconds:
            del dict_knifers[steamid]

